#include "UserAPI.h"

/*
 * NOTE: You are actually writing a Palm program even if you make no 
 * modifications to your easyC code!  Things like strcpy either don't 
 * exist or work differently than you are used to.  Get a good Palm
 * programming book if you need to know more than this :-)
 */
#include <PalmOS.h>  

/* all this is all optional up to the "Program 1" comment.  The code in the 
 * programs is mostly bogus so trash it when you have something real to put 
 * there.
 */

#define false 0
#define true !false
#define min(a, b) (((a) < (b)) ? (a) : (b))

#define DEBUG 0

#define DANGER 5
#define SLOWDOWN 20
#define OPENUP 30
#define SPEEDUP  60
#define INFINITY 99

#define SMALL 260
#define BIG 400 

#define MOTOR_L 1
#define MOTOR_R 2
#define ULTRA_L_INT 1
#define ULTRA_L_DATA 11
#define ULTRA_R_INT 2
#define ULTRA_R_DATA 12
#define ULTRA_M_INT 3
#define ULTRA_M_DATA 13
#define BUMPER_L 9
#define BUMPER_R 10
#define ENCODER_L 4
#define ENCODER_R 5

/* ----- The motor on the Right is oriented to standard
 * Left - Right
 * 0    - 255
 * 255  - 0
 * B=255,0 F=0,255 R=0,0 L=255,255
 */

char speed = 127;
int last_turn = -1;
int turn_count = 0;
 
void stop( void )
{ 
	speed = 0;
	SetMotor(MOTOR_L, 127);
	SetMotor(MOTOR_R, 127);
}

void forward(int _speed)
{
	if (_speed > 127) _speed = 127;
	if (_speed < -127) _speed = -127;
	speed = _speed;
	
	PrintToScreen("F(%d) \n", speed);
	if (DEBUG) return;
	
	SetMotor(MOTOR_L, 127-speed);
	SetMotor(MOTOR_R, 127+speed);
	Wait(50+speed);
}

void back(void)
{
	PrintToScreen("B(%d) ", speed);
	if (DEBUG) return;
	
	SetMotor(MOTOR_L, 127+speed);
	SetMotor(MOTOR_R, 127-speed);
	Wait(100+speed);
}

void left(int waiting)
{
	if (last_turn == 1)
	{
		if (++turn_count > 9) turn_count = 1;
	}
	else
		turn_count = 0;
	last_turn = 0;
	
	PrintToScreen("L(%d %d %d) \n", speed, last_turn, turn_count);
	if (DEBUG) return;
	
	SetMotor(MOTOR_L, 127+speed);
	SetMotor(MOTOR_R, 127+speed);
	if (turn_count > 0)
		waiting+=20*turn_count;
	Wait(waiting);
	forward(speed);
}

void right(int waiting)
{
	if (last_turn == 0)
	{
		if (++turn_count > 9) turn_count = 1;
	}
	else
		turn_count = 0;
	last_turn = 1;
	
	PrintToScreen("R(%d %d %d) \n", speed, last_turn, turn_count);
	if (DEBUG) return;
	
	SetMotor(MOTOR_L, 127-speed);
	SetMotor(MOTOR_R, 127-speed);
	if (turn_count > 0)
		waiting+=10*turn_count;
	Wait(waiting);
	forward(speed);
}

/*
 * Speed = 0..127
 * Turn Left == curve<0,  Turn Right == curve>0
 */
void move(int speed, int curve, int duration)
{
	int left, right;
	
	if (speed > 127)  speed = 127;
	if (speed < -127) speed = -127;
	if (curve > 127)  curve = 127;
	if (curve < -127) curve = -127;
	left = 127-speed-curve;
	if (left < 0) left = 0;
	if (left > 254) left = 254;
	right = 127+speed-curve;
	if (right < 0) right = 0;
	if (right > 254) right = 254;
	
	PrintToScreen("M(%d,%d:%d) \n", speed, curve, duration);
	if (DEBUG) return;
	
	SetMotor(MOTOR_L, left);
	SetMotor(MOTOR_R, right);
	Wait(duration);
}

void back_left(void)
{
	move(-75,-20,70);
	move(50, 0, 0);
}

void back_right(void)
{
	move(-75,20,70);
	move(50, 0, 0);
}

int bumpers( void )
{
	if (GetDigitalInput(BUMPER_L) == 0)
	{
		PrintToScreen("Bumped<\n");
		back_right();
		return true;
	}
	else if (GetDigitalInput(BUMPER_R) == 0)
	{
		PrintToScreen("Bumped>\n");
		back_left();
		return true;
	}
	
	return false;
}

//------------------------------------------------------------------------------
// Program 1
//------------------------------------------------------------------------------

char *prog1init( void )
{
	char *name = (char *) MemPtrNew(10);
	StrCopy(name, "Rover");
	return name;
}

int prog1 ( void )
{
	int uL = 1;
	int uR = 2;
	int uM = 3;
	int diff;
	int iters = -1;
	
	if (DEBUG != 1) move(127,0,0);
	while ( iters && VexRunning() )
	{
		bumpers();
		uM = ReadUltrasonic(ULTRA_M_INT, ULTRA_M_DATA);
		if (uM < SLOWDOWN)
		{
			if (uM < DANGER)
			{
				move(-75, 0, 70);
				move(0, 0, 0);
			}
			else
				move(50+uM, 0, 0);
			uL = ReadUltrasonic(ULTRA_L_INT, ULTRA_L_DATA);
			uR = ReadUltrasonic(ULTRA_R_INT, ULTRA_R_DATA);
			diff = uR-uL;
			if (diff>0 && diff<20) diff = 20;
			if (diff<0 && diff>-20) diff = -20;
			if (min(uL, uR) < SLOWDOWN)
				move(-75, diff, 50);
			else 
				move(50+min(uL,uR), diff, 0);
			
		}
		/* --- doing nothing isn't right...
		else if (uM == INFINITY)
		{
			// Infinities can be spurious readings from oblique angles to a wall.
			// Need to check side sonics to see if we should turn away from it.
		}
		*/
		else
		{
			if (bumpers()) continue;
			uL = ReadUltrasonic(ULTRA_L_INT, ULTRA_L_DATA);
			if (uM == INFINITY && uL == INFINITY) continue;
			if (bumpers()) continue;
			uR = ReadUltrasonic(ULTRA_R_INT, ULTRA_R_DATA);
			if (uM == INFINITY && uR == INFINITY) continue;
			if (bumpers()) continue;
			diff = uR-uL;
			move(50+uM, diff, 0);
		}
		
		
		/*
		// If uL or uR == 99 it may be a wall at an angle rather than a 
		// large open space.  Treat this differently by examining the 
		// value of the other sensor first.
		diff = uL - uR;
		if (uL < DANGER)
			back_right();
		else if (uR < DANGER)
			back_left();
		if (diff > OPENUP)
			move(speed,-50,1);
		else if (diff < -OPENUP)
			move(speed,50,1);
		else if (uL < SLOWDOWN || uR < SLOWDOWN)
		{
			turn_count = 0;
			last_turn = -1;
			move(50+min(uL,uR),-diff,1);
			//forward(50);
		}
		else if (uL > SPEEDUP && uR > SPEEDUP)
		{
			turn_count = 0;
			last_turn = -1;
			move(50+min(uL,uR),0,1);
			//forward(127);
		}
		else  // SLOWDOWN <= uL and uR <= SPEEDUP
		{
			turn_count = 0;
			last_turn = -1;
			move(50+min(uL,uR),0,1);
		}		
		*/
	}
	move(0, 0, 0); // Stop motors
	
	if (!iters) PrintToScreen("Program Stopped\n");
	
	return 0;
}

//------------------------------------------------------------------------------
// Program 2
//------------------------------------------------------------------------------

char *prog2init( void )
{
	char *name = (char *) MemPtrNew(10);
	StrCopy(name, "Table");
	return name;
}

int prog2( void )
{
	unsigned long encoder, etest;
	unsigned char sound, test;
	int running = true;
	
	StartUltrasonic(ULTRA_M_INT, ULTRA_M_DATA);
	test = sound = GetUltrasonic(ULTRA_M_INT, ULTRA_M_DATA);
	PrintToScreen("s=%d\n", sound);
	PresetEncoder ( ENCODER_L, 0 ) ;
	StartEncoder ( ENCODER_L) ;
	move(35,-4 ,0);  // forward seems to pull to the right
	while (running && test<sound+5)
	{
		test = GetUltrasonic(ULTRA_M_INT, ULTRA_M_DATA);
		PrintToScreen("st=%d\n", test);
		running = VexRunning();
	}
	encoder = GetEncoder ( ENCODER_L) ; // Get stop value before actually stopped
	move(0,0,0);
	PrintToScreen("e=%ld\n", encoder);
	if (!running) return 0;
	MakeSound(1500, 200);
	
	PresetEncoder ( ENCODER_L, 0 ) ;
	etest = GetEncoder(ENCODER_L);
	PrintToScreen("et=%ld\n", etest);
	move(-35,-4,0);
	if (encoder < 8) encoder = 8;  // Prevent "infinite" loop w/unsigned negative number
	while (VexRunning() && etest < encoder-8) // fudge off the encoder value to compensate for slop in the readings
	{
		etest = GetEncoder(ENCODER_L);
		PrintToScreen("et=%ld\n", etest);
	}
	move(0,0,0);
	StopEncoder( ENCODER_L);
	
	return 0;
}

//------------------------------------------------------------------------------
// Program 3
//------------------------------------------------------------------------------

char *prog3init( void )
{
	char *name = (char *) MemPtrNew(10);
	StrCopy(name, "Spiral");
	return name;
}

int prog3( void )
{
	int iters;
	int sound = 2900;
	
	if (DEBUG != 1) forward(70);
	iters = 127-70;
	while ( iters>0 && VexRunning() )
	{
		MakeSound(1000+iters*10, 90);
		move(50, iters, 300-3*iters);
		iters -= 5;
	}
	forward(0);
	return 0;

	// Alternate version w/o using move() function
	PrintToScreen("Spiral\n");

	SetMotor(MOTOR_R,200);
	for (iters=150; iters>100; iters-=5, sound-=75)
	{
		MakeSound(sound, 90);
		SetMotor(MOTOR_L, 50);
		Wait(30);
		SetMotor(MOTOR_L,iters);
		Wait(50);
	}
	SetMotor(MOTOR_L, 50);
	Wait(30);
	forward(0);
	return 0;
}

//------------------------------------------------------------------------------
// Program 4
//------------------------------------------------------------------------------

char *prog4init( void )
{
	char *name = (char *) MemPtrNew(10);
	StrCopy(name, "Turn Times");
	return name;
}

int prog4( void )
{
	static long unsigned int waiting = 0;
	
	
	//speed = 0;  // Stop after turn is done
	//right(waiting);
	
	SetMotors(50, 50);
	Wait(waiting);
	SetMotors(127, 127);
	
	waiting += 50;
	return 0;
}	
	
//------------------------------------------------------------------------------
// Program 5
//------------------------------------------------------------------------------

char *prog5init( void )
{
	char *name = (char *) MemPtrNew(10);
	StrCopy(name, "Test");
	return name;
}

int prog5( void)
{
	//static unsigned char led_state = 0;
    //Char buffer[100];
    unsigned char u1, b;
    
    while (VexRunning())
    {
	    b = GetDigitalInput(8);
	    PrintToScreen("b=%d\n", b);
    }
    return 0;
    
    RunProgram(5);
    return 0;
    
    b = ReadUltrasonic(1, 11);
    PrintToScreen("b=%d\n", b);
    return 0;
    
    /*
    PrintToScreen("> IO_Init");
    IO_Initialization();
    PrintToScreen("<\n");
    */
        
    /*  Test Code...
    PrintToScreen("> %d ... ", led_state);
	SetDigitalOutput(16, led_state);
	led_state = 1 - led_state;
    PrintToScreen("<\n");
    */
    PrintToScreen("> Get_Dig ");
	b = GetDigitalInput(10);
    PrintToScreen("b=0x%x<\n", b);
        
    PrintToScreen("> Start");
	StartUltrasonic(1, 11);
    PrintToScreen("<\n");
        
	PrintToScreen("> Get_Ult ");
	u1 = GetUltrasonic(1, 11);
	PrintToScreen("u1=0x%x<\n", u1);
	
	return 0;
}

